#ifndef __EpicDATA__
#define __EpicDATA__
/*****************************************************************************
 * Copyright (C) 2009-2024   this file is part of the NPTool Project       *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Audrey Chatillon  contact address: audrey.chatillon@cea.fr                        *
 *                                                                           *
 * Creation Date  : décembre 2024                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class hold Epic Raw data                                    *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *   
 *                                                                           *
 *****************************************************************************/

// STL
#include <vector>
using namespace std;

// ROOT
#include "TObject.h"

class TEpicData : public TObject {
  //////////////////////////////////////////////////////////////
  // data members are hold into vectors in order 
  // to allow multiplicity treatment
  private: 
    vector<UShort_t>   fEpic_AnodeNbr;
    vector<Double_t>   fEpic_Q1;
    vector<Double_t>   fEpic_Q2;
    vector<Double_t>   fEpic_Time;
    vector<Double_t>   fEpic_Qmax;
    vector<Bool_t>     fEpic_isFakeFission;
    vector<Double_t>   fEpic_Time_HF;
    vector<Double_t>   fEpic_ToF;
  
    vector<Double_t>   fEpic_FF_posZ;
    vector<Double_t>   fEpic_FF_DE;
    vector<Double_t>   fEpic_Electron_posZ;
    vector<Double_t>   fEpic_Electron_DE;


  //////////////////////////////////////////////////////////////
  // Constructor and destructor
  public: 
    TEpicData();
    ~TEpicData();
    

  //////////////////////////////////////////////////////////////
  // Inherited from TObject and overriden to avoid warnings
  public:
    void Clear();
    void Clear(const Option_t*) {};
    void Dump() const;


  //////////////////////////////////////////////////////////////
  // Getters and Setters
  // Prefer inline declaration to avoid unnecessary called of 
  // frequently used methods
  // add //! to avoid ROOT creating dictionnary for the methods
  public:
    //////////////////////    SETTERS    ////////////////////////
    inline void SetAnodeNbr(const UShort_t& AnodeNbr){fEpic_AnodeNbr.push_back(AnodeNbr);}//!
    inline void SetQ1(const Double_t& Q1){fEpic_Q1.push_back(Q1);}//!
    inline void SetQ2(const Double_t& Q2){fEpic_Q2.push_back(Q2);}//!
    inline void SetQmax(const Double_t& Qmax){fEpic_Qmax.push_back(Qmax);}//!
    inline void SetTime(const Double_t& Time){fEpic_Time.push_back(Time);}//!
    inline void SetTimeHF(const Double_t& Time){fEpic_Time_HF.push_back(Time);}//!
    inline void SetToF(const Double_t& ToF){fEpic_ToF.push_back(ToF);}//!
    inline void SetFakeFissionStatus(const Bool_t& isFF){fEpic_isFakeFission.push_back(isFF);}//!

    inline void SetFFPosZ(const Double_t& z){fEpic_FF_posZ.push_back(z);}//!
    inline void SetFFDE(const Double_t& DE){fEpic_FF_DE.push_back(DE);}//!
    inline void SetElectronPosZ(const Double_t& z){fEpic_Electron_posZ.push_back(z);}//!
    inline void SetElectronDE(const Double_t& DE){fEpic_Electron_DE.push_back(DE);}//!

    //////////////////////    GETTERS    ////////////////////////
    inline UShort_t GetMultiplicity() const                   {return fEpic_AnodeNbr.size();}
    inline UShort_t GetAnodeNbr(const unsigned int &i) const  {return fEpic_AnodeNbr[i];}//!
    inline Double_t GetQ1(const unsigned int &i) const        {return fEpic_Q1[i];}//!
    inline Double_t GetQ2(const unsigned int &i) const        {return fEpic_Q2[i];}//!
    inline Double_t GetQmax(const unsigned int &i) const      {return fEpic_Qmax[i];}//!
    inline Double_t GetTime(const unsigned int &i) const      {return fEpic_Time[i];}//!
    inline Double_t GetTimeHF(const unsigned int &i) const    {return fEpic_Time_HF[i];}//!
    inline Double_t GetToF(const unsigned int &i) const       {return fEpic_ToF[i];}//!
    inline UShort_t GetHFMultiplicity() const                 {return fEpic_Time_HF.size();}//!
    inline Bool_t GetFakeFissionStatus(const unsigned int &i) const  {return fEpic_isFakeFission[i];}//!

    inline Double_t GetFFposZ(const unsigned int &i) const {return fEpic_FF_posZ[i];}//!
    inline Double_t GetFFDE(const unsigned int &i) const   {return fEpic_FF_DE[i];}//!
    inline Double_t GetElectronposZ(const unsigned int &i) const {return fEpic_Electron_posZ[i];}//!
    inline Double_t GetElectronDE(const unsigned int &i) const   {return fEpic_Electron_DE[i];}//!

  //////////////////////////////////////////////////////////////
  // Required for ROOT dictionnary
  ClassDef(TEpicData,1)  // EpicData structure
};

#endif
